from pyspark.sql import SparkSession
from pyspark.sql.functions import *

spark = SparkSession.builder.master('local').appName('Trazeni Muzicar').getOrCreate()

#uctavamo tabelu
sastav_benda = spark.read.csv('sastav_benda.csv', inferSchema = True)\
    .withColumnRenamed('_c0', 'idBenda')\
    .withColumnRenamed('_c1', 'idMuzicara')\
    .withColumnRenamed('_c2', 'datumOd')\
    .withColumnRenamed('_c3', 'datumDo')
# Pretvaramo u promenljivu datum
sastav_benda = sastav_benda.withColumn('datumDo', to_date(col('datumDo')))
# Cistimo null vrednosti sa danasnjim datumom
sastav_benda = sastav_benda.withColumn('datumDo', when(col('datumDo').isNull(), current_date()).otherwise(col('datumDo')))
# Pretvaramo u promenljivu datum
sastav_benda = sastav_benda.withColumn('datumOd', to_date(col('datumOd')))

#sastav_benda.show()

#Iscitavamo sve festivale
lineup_festivala = spark.read.csv('lineup_festivala.csv', inferSchema = True)\
    .withColumnRenamed('_c0', 'idFestivala')\
    .withColumnRenamed('_c1', 'godina')\
    .withColumnRenamed('_c2', 'idBenda')\
    .withColumnRenamed('_c3', 'dan')

#lineup_festivala.show()

#Iscitavamo muzicare
muzicari = spark.read.csv('muzicari.csv', inferSchema = True)\
    .withColumnRenamed('_c0', 'idMuzicara')\
    .withColumnRenamed('_c1', 'ime')\
    .withColumnRenamed('_c2', 'prezime')

#muzicari.show()

#Uzimamo poslednji dan na svim festivalima
lineup_festivala_max_dan = lineup_festivala.groupBy(col('idFestivala'), col('godina')).agg(max(col('dan')))

#lineup_festivala_max_dan.show()

#Spajamo sa danima koje smo malopre izvukli kako bi dobili sada i idBenda koji nam je potreban
lineup_festivala_max_dan = lineup_festivala_max_dan.join(lineup_festivala)\
    .filter((lineup_festivala_max_dan.godina == lineup_festivala.godina) & (lineup_festivala_max_dan.idFestivala == lineup_festivala.idFestivala))

lineup_festivala_max_dan.show()

#Spajamo sastav benda i festivale da bi dobili muzicare koji su svirali na festivalima
svi_muzicari_festivali = lineup_festivala_max_dan.join(sastav_benda)\
    .filter((lineup_festivala_max_dan.idBenda == sastav_benda.idBenda) & ((lineup_festivala_max_dan.godina >= year(sastav_benda.datumOd)) & (lineup_festivala_max_dan.godina <= year(sastav_benda.datumDo))))

# svi_muzicari_festivali.show()

#Sada kada imamo sve sto nam je potrebno sada mozemo da prebrojimo dane po idMuzicara da bi znali koji muzicar ima najvise nastupa
svi_muzicari_festivali = svi_muzicari_festivali.groupBy(col('idMuzicara')).agg(count(col('dan'))).alias('brojFestivala')

# svi_muzicari_festivali.show()

#Uzimamo muzicara sa najvise nastupa
svi_muzicari_festivali = svi_muzicari_festivali.groupBy(col('idMuzicara')).agg(max(col('brojFestivala')))

# svi_muzicari_festivali.show()

#Spajamo muzicara sa najvise nastupa sa imenom i prezimenom zbog konacnog resenja zadatka
resenje = svi_muzicari_festivali.join(muzicari)\
    .filter(svi_muzicari_festivali.idMuzicara == muzicari.idMuzicara)

# resenje.show()

#Stampamo u csv fajl resenje
resenje.write.mode('overwrite').csv('trazeniMuzicar.csv')