from pyspark.sql import SparkSession
from pyspark.sql.functions import *

spark = SparkSession.builder.master('local').appName('Trazeni muzicar').getOrCreate()

lineup_festivala = spark.read.csv('festivali/lineup_festivala.csv', inferSchema=True)\
    .withColumnRenamed('_c0', 'idFestivala')\
    .withColumnRenamed('_c1', 'godina')\
    .withColumnRenamed('_c2', 'idBenda')\
    .withColumnRenamed('_c3', 'dan')

# lineup_festivala.show()

muzicari = spark.read.csv('festivali/muzicari.csv', inferSchema=True)\
    .withColumnRenamed('_c0', 'idMuzicara')\
    .withColumnRenamed('_c1', 'ime')\
    .withColumnRenamed('_c2', 'prezime')

# muzicari.show()

sastav_benda = spark.read.csv('festivali/sastav_benda.csv', inferSchema=True)\
    .withColumnRenamed('_c0', 'idMuzicara')\
    .withColumnRenamed('_c1', 'idBenda')\
    .withColumnRenamed('_c2', 'datum1')\
    .withColumnRenamed('_c3', 'datum2')

#bendovi koji su svirali na poslednjem danu festivala
lineup_festivala_max = lineup_festivala.groupBy(col('godina')).agg(max(col('dan')).alias('maxDan'))
lineup_festivala = lineup_festivala.groupBy(col('godina'),col('idBenda')).agg(max(col('dan')).alias('maxDan'))

# lineup_festivala.show()
# lineup_festivala_max.show()

#bendovi sa najvise nastupa na poslednjim danima
lineup_festivala_max = lineup_festivala_max.join(lineup_festivala, (lineup_festivala_max.godina == lineup_festivala.godina) & (lineup_festivala_max.maxDan == lineup_festivala.maxDan))

lineup_festivala_max.show()


sastav_benda = sastav_benda.withColumn('datum1', year(col('datum1')))
sastav_benda = sastav_benda.withColumn('datum2', when(col('datum2') == 'NULL', year(current_date())).otherwise(year('datum2')))

# sastav_benda.show()

#muzicari sa najvise nastupa na poslednjem danu
sastav_benda = sastav_benda.join(lineup_festivala_max, (sastav_benda.godina.between(sastav_benda.datum1, sastav_benda.datum2)) & (sastav_benda.idBenda == lineup_festivala_max.idBenda))
sastav_benda = sastav_benda.groupBy('idMuzicara').agg(count('*').alias('brojPojavljivanja')).desc()

sastav_benda = sastav_benda.join(muzicari, 'idMuzicara').select(col('ime'), col('prezime'))

sastav_benda.write.mode('overwrite').csv('trazeniMuzicar.csv', header=True)