

from pyspark import SparkContext, SparkConf
from datetime import datetime
import os
import shutil
import csv

INPUT_DIR = "festivali"
OUTPUT_DIR = "trazeniMuzicar"

if os.path.exists(OUTPUT_DIR):
	shutil.rmtree(OUTPUT_DIR)

def ucitajRddCsv(filename):
	filename = os.path.join(INPUT_DIR, filename)
	return sc.textFile(filename).mapPartitions(lambda x: csv.reader(x))
	
def poslednjiDanFesta(x):
	# (idFesta, Iterable)
	idFesta, iterable = x
	
	# pronalazak maks dana
	for (dan, godina, idBenda) in sorted(iterable):
		pass
		
	lista = []
	# posto jednog dana moze svirati vise bendova
	for (dan1, godina1, idBenda1) in iterable:
		if dan == dan1:
			lista.append( (idBenda, (dan, godina)) )
		
	return lista

def mapSastavi(inp):
	idMuzicara, idBenda, datum1, datum2 = inp
	
	godina1 = datum1
		
	if datum2 != "NULL":
		godina2 = datetime.strptime(inp, "%Y-%M-%d").date().year
	
	return (idBenda, (idMuzicara, godina1, godina2, 0))
	
def svirao(godinaFesta, godina1, godina2):
	if godinaFesta == godina1:
		return True
	
	if godinaFesta > godina1 and (godina2 == "NULL" or godina2 > godinaFesta):
		return True
		
	return False
	
def filterSviraUBendu(inp):
	# (idBenda, ((dan, godina)), (idMuzicara, godina1, godina2, brojNastupa))
	idBenda, inp1 = inp
	inp1, inp2 = inp1
	dan, godina = inp1
	idMuzicara, godina1, godina2, brojNastupa = inp2
	
	if svirao(godina, godina1, godina2):
		return (idMuzicara, brojNastupa)

conf = SparkConf().setMaster("local").setAppName("kolokvijum")
sc = SparkContext(conf = conf)

sc.setLogLevel("ERROR")

lineupRdd = ucitajRddCsv("lineup_festivala.csv").map(lambda x: (x[0], (int(x[3]), int(x[1]), x[2])))
lineupGroup = lineupRdd.groupByKey()
lineupPoslednjiDan = lineupGroup.flatMap(poslednjiDanFesta)
#print(lineupPoslednjiDan.take(2))
# (idBenda, (dan, godina))

sastavBendaRdd = ucitajRddCsv("sastav_benda.csv").map(mapSastavi)
#print(sastavBendaRdd.take(4))
# (idBenda, (idMuzicara, godina1, godina2, brojNastupa))

joined = lineUpPoslednjiDan.join(sastavBendaRdd)
# (idBenda, ((dan, godina)), (idMuzicara, godina1, godina2, brojNastupa))
sviraliPoslednjiDan = joined.filter(filterSviraUBendu)
# (idMuzicara, brojNastupa)
sviraliPoslednjiDanUkupno = sviraliPoslednjiDan.reduceByKey(lambda x, y: x + y)
sviraliPoslednjiDanUkupno = sviraliPoslednjiDanUkupno.map(lambda x: (x[1], x[0])).sortByKey()
# Odredjen broj sviranja na festivalima poslednjeg dana svakog muzicara
# (brojNastupa, idMuzicara)

najcesceSvirao = sviraliPoslednjiDanUkupno.first().map(lambda x: (x[1], x[0]))
# (idMuzicara, brojNastupa)

muzicariRdd = ucitajRddCsv("muzicari.csv").map(lambda x: (x[0],(x[1], x[2])))
# (idMuzicara, (ime, prezime))

najcesceSvirao = najcesceSvirao.join(muzicariRdd)
# (idMuzicara, (brojNastupa, (ime, prezime)))
najcesceSvirao = najcesceSvirao.map(lambda x: (x[1][1][0], x[1][1][1], x[1][0]))
najcesceSvirao.saveAsTextFile(OUTPUT_DIR)




